// Código de funcionamento Snake
// Traduzido e adaptado por Usinainfo
// www.usinainfo.com.br

#include "LedControl.h"
#include "pitches.h"

// Constantes da Madriz de LED 8x8
const byte DIN = 5;
const byte CLK = 3;
const byte CS = 4;
LedControl lc = LedControl(DIN, CLK, CS, 1); // 1 = numero de controladores

//Constante dos Push Buttons
const byte direita = 13;
const byte esquerda = 12;
const byte cima = 11;
const byte baixo = 10;

const byte buzzer = 9; //Constante do Buzzer

//Demais variáveis de programação
int had[8][8] = {0};

// Variáveis de posição
byte xHad = 4;
byte yHad = 5;
byte xPotrava = 0;
byte yPotrava = 0;

byte smer = 0;
int krok = 1;
long CasZmeny = 0;
int rychlost = 500;
byte skore = 0;

boolean snezeno = false;
boolean zvuk = true;
boolean probehlo = false;

// Tons de melodia declarados
int melody[] = {
  NOTE_C4, NOTE_G3, NOTE_G3, NOTE_A3, NOTE_G3, 0, NOTE_B3, NOTE_C4
};
int noteDurations[] = {
  4, 8, 8, 4, 4, 4, 4, 4
};

//Declarando números e posição
int Cislo[11][7] = {
  {B11100000, B01000000, B01000000, B01000000, B01000000, B11000000, B01000000}, //1
  {B11111000, B01000000, B00100000, B00010000, B00001000, B10001000, B01110000}, //2
  {B01110000, B10001000, B00001000, B00010000, B00100000, B00010000, B11111000}, //3
  {B00010000, B00010000, B11111000, B10010000, B01010000, B00110000, B00010000}, //4
  {B01110000, B10001000, B00001000, B00001000, B11110000, B10000000, B11111000}, //5
  {B01110000, B10001000, B10001000, B11110000, B10000000, B01000000, B00110000}, //6
  {B00100000, B00100000, B00100000, B00010000, B00001000, B10001000, B11111000}, //7
  {B01110000, B10001000, B10001000, B01110000, B10001000, B10001000, B01110000}, //8
  {B01100000, B00010000, B00001000, B01111000, B10001000, B10001000, B01110000}, //9
  {B01110000, B10001000, B11001000, B10101000, B10011000, B10001000, B01110000}, //0
  {B00000000, B00000000, B00000000, B00000000, B00000000, B00000000, B00000000}, //mezera
};

void setup() {
  Serial.begin(9600);
  Serial.println("Iniciar Jogo Snake");

  randomSeed(analogRead(A0));
  GenerujPotravu();

  pinMode(esquerda, INPUT_PULLUP);
  pinMode(direita, INPUT_PULLUP);
  pinMode(cima, INPUT_PULLUP);
  pinMode(baixo, INPUT_PULLUP);

  pinMode(buzzer, OUTPUT);

  //Inicio de exposição 8x8
  lc.shutdown(0, false);
  lc.setIntensity(0, 1);
  lc.clearDisplay(0);
}

void loop() {
  if ((digitalRead(esquerda) == false) && (smer != 0)) smer = 1;
  if ((digitalRead(direita) == false) && (smer != 1)) smer = 0;
  if ((digitalRead(cima) == false) && (smer != 3)) smer = 2;
  if ((digitalRead(baixo) == false) && (smer != 2)) smer = 3;

  if ((millis() - CasZmeny) > rychlost) {
    CasZmeny = millis();

    VymazHada(had);
    Skore();
    Pohyb();

    if (xHad == xPotrava && yHad == yPotrava) snezeno = true;
    if (had[xHad][yHad] != 0) GameOver();

    krok++;
    had[xHad][yHad] = krok;
  }
  Vykres(had, xPotrava, yPotrava);
}

//Função Skore
void Skore() {
  if (snezeno == true) {
    snezeno = false;
    GenerujPotravu();

    skore++;
    tone(buzzer, NOTE_C5, 200);
    rychlost -= 20;

    Serial.print ("Pontuação/Contagem: ");
    Serial.println (skore);
  }
}

void Vykres(int matice[8][8], byte x, byte y) {
  for (int i = 0; i < 8; i++) {
    for (int j = 0; j < 8; j++) {
      if ((matice[i][j] >= 1) || ((i == x) && (j == y) )) lc.setLed(0, i, j, true);
      else lc.setLed(0, i, j, false);
    }
  }
}

void VymazHada(int matice[8][8]) {
  for (int i = 0; i < 8; i++) {
    for (int j = 0; j < 8; j++) {
      if (matice[i][j] == (krok - skore)) matice[i][j] = 0;
    }
  }
}

void Pohyb() {
  switch (smer) {
    case 0:
      xHad++;
      break;
    case 1:
      xHad--;
      break;
    case 2:
      yHad++;
      break;
    case 3:
      yHad--;
      break;
  }
  if (xHad == 8) xHad = 0;
  if (yHad == 8) yHad = 0;
  if (xHad == 255) xHad = 7;
  if (yHad == 255) yHad = 7;
}

void GenerujPotravu() {
  lc.setLed(0, xPotrava, yPotrava, false);
  do {
    xPotrava = random(0, 8);
    yPotrava = random(0, 8);
  } while (had[xPotrava][yPotrava] != 0 );
}

void VykresCislo(byte cislo) {
  if (cislo == 0) cislo = 10;
  for (int col = 0; col < 7; col++) {
    lc.setColumn(0, col, Cislo[cislo - 1][col]);
  }
}

void GameOver() {
  Serial.println ("Fim de Jogo");
  for (int row = 0; row < 8; row++) {
    for (int col = 0; col < 8; col++) {
      if (zvuk == true) {
        int noteDuration = 1000 / noteDurations[col];
        tone(buzzer, melody[col], noteDuration);
        int pauseBetweenNotes = noteDuration * 1.30;
        delay(pauseBetweenNotes);
        noTone(buzzer);
      }
      lc.setLed(0, col, row, true);
      delay(25);
    }
    zvuk = false;
  }
  for (int row = 0; row < 8; row++) {
    for (int col = 0; col < 8; col++) {
      lc.setLed(0, col, row, false);
      delay(25);
    }
  }
  do {
    VykresCislo((skore / 10) % 10);
    delay(200);
    VykresCislo(skore % 10);
    delay(200);
    VykresCislo(11);
    delay(1000);
  } while (true);
}
